const axios = require('axios');
const {
    default: makeWASocket,
    DisconnectReason,
    useMultiFileAuthState,
    delay
} = require('@adiwajshing/baileys');
const P = require('pino');
const fs = require("fs");
const express = require('express');
const qrcode = require('qrcode')
const figlet = require('figlet');
const cron = require('node-cron');
const phpExpress = require('php-express')({ binPath: 'php' });
const { Boom } = require('@hapi/boom');
const { color } = require('./lib/color');
const { Server } = require("socket.io");
const { createServer } = require('node:http');
const { setPassword, setSSIDName, getSSIDInfo, rebootRouter } = require("./lib/wifi");

const config = JSON.parse(fs.readFileSync("./config.json"));
// global db
global.users = JSON.parse(fs.readFileSync("./database/users.json"));
let cronConfig = JSON.parse(fs.readFileSync('./database/cron.json'));


const msgHandler = require('./message/idoy');

const jwt = require('jsonwebtoken');
const cookieParser = require('cookie-parser');

console.log(color(figlet.textSync('HALLOBAY', {
    font: 'Standard',
    horizontalLayout: 'default',
    vertivalLayout: 'default',
    whitespaceBreak: false
}), 'cyan'));
console.log(color('[ BayuLingga ]'));

let conn;
let cronTask = null;
const PORT = process.env.PORT || 3100;

const app = express();

app.use(express.json());
app.use(express.urlencoded({ extended: true }));
app.use(cookieParser('rweutkhdrt'))
app.set('views', 'views');
app.use(express.static('static'));

app.use(async (req, res, next) => {
    if (req.url.match(/.+\.php/)) return next();
    if (!req.cookies.token && !["/login"].includes(req.url)) return res.redirect("/login");
    if (req.cookies.token) {
        const decoded = decode(req.cookies.token);
        if (decoded) {
        } else {
            res.cookie("token", "", {
                httpOnly: true,
                maxAge: 0
            });
            return res.redirect("/login");
        }
    }
    next();
});

app.get('/login', (req, res) => {
    res.sendFile('sb-admin/login.html', { root: 'views' });
});

app.get("/logout", (req, res) => {
    res.cookie("token", "", {
        httpOnly: true,
        maxAge: 0
    });
    return res.redirect("/login");
});

app.post('/login', (req, res) => {
    if (req.body.username != config.username || req.body.password != config.password) return res.redirect('/login');
    const token = jwt.sign({ username: req.body.username }, config.jwt, { expiresIn: '7d' });
    res.cookie("token", token, {
        httpOnly: true,
        maxAge: 6.048e+8
    });
    res.redirect('/');
});

app.get('/', (req, res) => {
    res.sendFile('sb-admin/index.html', { root: 'views' })
});

app.get('/start', (req, res) => {
    if (!conn) connect();
    res.json({ message: !!conn ? 'bot is online' : 'starting bot' });
});

app.get('/stop', (req, res) => {
    if (!!conn) {
        conn.end();
        conn = null;
    }
    res.json({ message: 'Bot is offline' });
});

app.get('/stats', (req, res) => {
    res.json({
        users: users.length,
        botStatus: !!conn
    });
});

app.get('/cron', (req, res) => {
    if (req.query.json) return res.json({ data: cronConfig });
    res.sendFile(`sb-admin/cron.html`, { root: 'views' })
});

app.get('/users', (req, res) => {
    res.sendFile(`sb-admin/users.html`, { root: 'views' })
});

app.get('/broadcast', (req, res) => {
    res.sendFile(`sb-admin/broadcast.html`, { root: 'views' })
})

app.get('/api/:category', (req, res) => {
    const { category } = req.params;
    return res.json({ data: category == 'users' ? users : [] })
});


app.get('/api/reboot/ssid/:id', async (req, res) => {
    try {
        await rebootRouter(req.params.id);
        return res.redirect("/users");
    } catch (e) {
        console.log(e);
        res.json({
            status: 500,
            message: "Internal server error"
        });
    }
})

app.post('/api/update/ssid/:id', async (req, res) => {
    try {
        const { ssid_name, ssid_password, transmit_power } = req.body;
        if (ssid_name || ssid_password || transmit_power) {
            await axios.post(config.genieacsBaseUrl + "/devices/" + req.params.id + "/tasks?connection_request", {
                name: 'setParameterValues',
                parameterValues: [
                    ...(transmit_power ? [["InternetGatewayDevice.LANDevice.1.WLANConfiguration.1.TransmitPower", transmit_power, "xsd:string"]] : []),
                    ...(ssid_name ? [["InternetGatewayDevice.LANDevice.1.WLANConfiguration.1.SSID", ssid_name, "xsd:string"]] : []),
                    ...(ssid_password ? [["InternetGatewayDevice.LANDevice.1.WLANConfiguration.1.PreSharedKey.1.PreSharedKey", ssid_password, "xsd:string"]] : [])
                ]
            })
        }
        res.redirect("/users")
    } catch (e) {
        console.log(e)
        res.json({
            status: 500,
            message: "Internal server error"
        });
    }
})

app.post('/api/update/cron', (req, res) => {
    const { schedule, message } = req.body;
    cronConfig = JSON.stringify({
        schedule,
        message
    });
    fs.writeFileSync('./database/cron.json', cronConfig);
    if (!!cron) {
        cron.stop();
        startCron();
    }
    res.redirect("/cron");
});

app.post("/api/broadcast", async (req, res) => {
    if (!conn) return res.json({
        status: 500,
        message: "the server is not connected to WhatsApp"
    });
    broadcast(req.body.text)
    return res.redirect("/broadcast");
})

app.post('/api/:category/:id?', async (req, res) => {
    const { category, id } = req.params;
    switch (category) {
        case 'users':
            if (id) {
                users = users.map(user => (user.id == id ? { ...req.body, id: user.id } : user));
            } else {
                //const resppp = await addpppoe(req.body.pppoe_username, req.body.pppoe_password, req.body.pppoe_profile)
                //if (['', 'ERROR !!! AKUN PPPOE SUDAH TERDAFTAR', 'ERROR !!! KESALAHAN PENULISAN PROFILE'].includes(resppp)) return res.json({ message: resppp})
                users.push({ ...req.body, id: (users[users.length - 1]?.id || 0) + 1 });
            }
            saveUser();
            break;
        default:
            return res.status(404).json({ message: 'category not found' });
            break;
    }
    return res.redirect('/' + category);
    // return res.json({ message: id ? 'Successfully updated' : 'Successfully created' });
});

app.delete('/api/:category/:id', (req, res) => {
    const { category, id } = req.params;
    switch (category) {
        case 'users':
            users = users.filter(user => user.id != id);
            saveUser();
            break;
    }
    return res.json({ message: 'Successfully deleted' })
});


app.get("/send/:id/:text", async (req, reply) => {
    if (!conn) return reply.send({
        status: 500,
        message: "the server is not connected to WhatsApp"
    });
    try {
        if (req.params.id.endsWith("@g.us")) {
            let groups = Object.entries(await conn.groupFetchAllParticipating()).slice(0).map(entry => entry[1]);
            if (!groups.find(v => v.id === req.params.id)) return reply.send({
                status: 400,
                message: "Invalid group id"
            });
        } else if (req.params.id.endsWith("@c.us") || req.params.id.endsWith("@s.whatsapp.net")) {
            if (!(await conn.onWhatsApp(req.params.id))[0]) return reply.send({
                status: 400,
                message: "Invalid contact id"
            });
        } else return reply.send({
            status: 400,
            message: "Invalid id"
        });
        const send = await conn.sendMessage(req.params.id, {
            text: req.params.text
        });
        return reply.send({
            status: 200,
            message: `Success send message with text ${req.params.text}`,
            result: send
        });
    } catch (e) {
        return reply.send({
            status: 500,
            message: e
        });
    }
});


app.engine('php', phpExpress.engine);
app.set('view engine', 'php');
app.all(/.+\.php$/, phpExpress.router);

const server = createServer(app);
const io = new Server(server);

server.listen(PORT, async () => {
    if (fs.existsSync(`sessions/${config.sessionName}`)) connect();
    console.log('listening on port', PORT);
});


if (cronConfig.schedule?.length > 0) {
    startCron();
}

async function connect() {
    const { state, saveCreds: saveState } = await useMultiFileAuthState(`sessions/${config.sessionName}`)
    const idoy = makeWASocket({
        printQRInTerminal: true,
        logger: P({
            level: 'fatal'
        }),
        browser: ["ALIJAYA BOT", "safari", "1.0.0"],
        auth: state
    });
    idoy.multi = true
    idoy.nopref = false
    idoy.prefa = 'anjing'
    idoy.mode = 'public'

    idoy.ev.on('messages.upsert', async m => {
        if (!m.messages || !m.messages[0]?.message) return;
        msgHandler(idoy, m.messages[0], m);
    });

    idoy.ev.on('connection.update', async update => {
        const {
            connection,
            lastDisconnect
        } = update
        if (connection === 'open') {
            conn = idoy;
            console.log("connected");
            io.emit('message', 'connected');
        } else if (connection === 'close') {
            let reason = new Boom(lastDisconnect?.error)?.output.statusCode;
            if (reason === DisconnectReason.connectionReplaced) {
                console.log("Connection Replaced, Another New Session Opened, Please Close Current Session First");
                idoy.logout();
            } else if (reason === DisconnectReason.loggedOut) {
                console.log(`Device Logged Out, Please Scan Again`);
                fs.rmSync(`sessions/${config.sessionName}`, { recursive: true });
                conn = null;
            } else {
                console.log("reconnect");
                connect();
            }
        } else if (update.qr) {
            console.log("Please scan QR code");
            qrcode.toDataURL(update.qr, (err, url) => {
                io.emit('qr', url);
            });
        } else console.log(update);
    });

    idoy.ev.on('creds.update', saveState);
    return idoy;
}

function saveUser() {
    fs.writeFileSync('./database/users.json', JSON.stringify(users), 'utf-8');
}


function decode(token, secret = config.jwt) {
    try {
        return jwt.verify(token, secret);
    } catch {
        return undefined;
    }
}

async function broadcast(text) {
    for (let user of global.users) {
        conn.sendMessage(user.phone_number + "@s.whatsapp.net", {
            text
        });
        await delay(1000)
    }
}

function startCron() {
    cronTask = cron.schedule(cronConfig.schedule, async () => {
        users = users.map(user => {
            return {
                ...user,

            }
        })
        saveUser();
        for (let user of users) {
            await Promise.all([
                conn.sendMessage(user.phone_number + "@s.whatsapp.net", {
                    text: cronConfig.message
                }),
                delay(2000)
            ]);
        }
    })
}