const axios = require("axios")
const { fetchText, fetchJson } = require('../tools/fetcher')
const config = require('../config.json')


const getDeviceByIdentifier = async (identifier) => {
  let query = { _id: identifier };
  let queryUrl = `${config.genieacsBaseUrl}/devices?query=${encodeURIComponent(JSON.stringify(query))}`;
  let response = await axios.get(queryUrl);
  let devices = response.data;

  if (devices.length === 0) {
    query = { "VirtualParameters.pppUsername": identifier };
    queryUrl = `${config.genieacsBaseUrl}/devices?query=${encodeURIComponent(JSON.stringify(query))}&projection=_id`;
    response = await axios.get(queryUrl);
    devices = response.data;
  }

  if (devices.length === 0) {
    throw new Error(`No device found for ${identifier}.`);
  }

  return devices[0]._id;
};

const setPassword = (deviceId, newPassword) => {
  // Mengganti % menjadi %25 pada deviceId
  const encodedDeviceId = deviceId.replace(/%/g, '%25');

  return axios.post(`${config.genieacsBaseUrl}/devices/${encodedDeviceId}/tasks?connection_request`, {
    name: 'setParameterValues',
    parameterValues: [
      ["InternetGatewayDevice.LANDevice.1.WLANConfiguration.1.PreSharedKey.1.KeyPassphrase", newPassword, "xsd:string"]
    ]
  });
};

const setSSIDName = (deviceId, newName) => {
  // Mengganti % menjadi %25 pada deviceId
  const encodedDeviceId = deviceId.replace(/%/g, '%25');

  return axios.post(`${config.genieacsBaseUrl}/devices/${encodedDeviceId}/tasks?connection_request`, {
    name: 'setParameterValues',
    parameterValues: [
      ["InternetGatewayDevice.LANDevice.1.WLANConfiguration.1.SSID", newName, "xsd:string"]
    ]
  });
};
// const setSSIDName = (deviceId, newName) => {
//   // Mengganti % menjadi %25 pada deviceId
//   const encodedDeviceId = deviceId.replace(/%/g, '%25');

//   return axios.post(`${config.genieacsBaseUrl}/devices/${encodedDeviceId}/tasks?connection_request`, {
//     name: 'setParameterValues',
//     parameterValues: [
//       ["InternetGatewayDevice.LANDevice.1.WLANConfiguration.2.SSID", newName, "xsd:string"]
//     ]
//   });
// };


const ubahPass = async (identifier, newPassword) => {
  try {
    const deviceId = await getDeviceByIdentifier(identifier);
    await setPassword(deviceId, newPassword);

    return `Kata Sandi Berhasil Dirubah:\n\n==============\n${newPassword}\n==============\n\n${config.namabot}`;
  } catch (error) {
    console.error('Failed to change password:', error);
    return `Gagal Mengubah Password WiFi\n\nSilahkan Cek Format Password Atau Hubungi Admin\n\nTerimakasih\n\n${config.namabot}`;
  }
};

const ubahSSID = async (identifier, newSSID) => {
  try {
    const deviceId = await getDeviceByIdentifier(identifier);
    await setSSIDName(deviceId, newSSID);

    return `Nama Wifi Berhasil Dirubah:\n\n==============\n${newSSID}\n==============\n\n${config.namabot}`;
  } catch (error) {
    console.error('Failed to change SSID:', error);
    return `Gagal Mengubah Nama Wifi\n\nSilahkan Cek Format Nama Wifi Atau Hubungi Admin\n\nTerimakasih\n\n${config.namabot}`;
  }
};

const getSSIDInfo = async (identifier) => {
  try {
    // Tentukan query berdasarkan apakah identifier adalah device_id atau pppoe_username
    let query;
    if (identifier.includes(".")) {
      query = { "VirtualParameters.pppUsername._value": identifier }; // pppoe_username biasanya memiliki tanda titik
    } else {
      query = { _id: identifier }; // device_id biasanya berupa string hex (tidak ada tanda titik)
    }

    // Tampilkan query yang digunakan
    console.log("Query yang digunakan:", query);

    const queryUrl = `${config.genieacsBaseUrl}/devices?query=${encodeURIComponent(JSON.stringify(query))}`;

    const response = await axios.get(queryUrl);
    const data = response.data[0];

    if (!data) {
      throw new Error("Device not found");
    }

    return {
      uptime: data.VirtualParameters.uptimeDevice._value,
      ssidName: data.InternetGatewayDevice.LANDevice['1'].WLANConfiguration['1'].SSID._value,
      totalAssociation: data.InternetGatewayDevice.LANDevice['1'].WLANConfiguration['1'].TotalAssociations._value,
      transmitPower: data.InternetGatewayDevice.LANDevice['1'].WLANConfiguration['1'].TransmitPower._value
    };
  } catch (error) {
    console.error('Gagal mendapatkan informasi SSID:', error);
    throw error;
  }
};


const getDeviceInfo = async (identifier) => {
  try {
    // Mencoba menemukan perangkat berdasarkan ID perangkat
    let query = { _id: identifier };
    let queryUrl = `${config.genieacsBaseUrl}/devices?query=${encodeURIComponent(JSON.stringify(query))}`;
    console.log("Query yang digunakan:", query);

    let response = await axios.get(queryUrl);
    let devices = response.data;

    if (devices.length === 0) {
      // Jika tidak ditemukan, coba mencari berdasarkan pppUsername
      query = { "VirtualParameters.pppUsername": identifier };
      queryUrl = `${config.genieacsBaseUrl}/devices?query=${encodeURIComponent(JSON.stringify(query))}`;
      console.log("Query yang digunakan:", query);

      response = await axios.get(queryUrl);
      devices = response.data;
      if (devices.length === 0) {
        throw new Error(`No device found for ${identifier}.`);
      }
    }

    const deviceId = devices[0]._id;
    console.log("Device ID ditemukan:", deviceId);

    const generateUrl = (projection) => `${config.genieacsBaseUrl}/devices?query=${encodeURIComponent(JSON.stringify({ _id: deviceId }))}&projection=${projection}`;

    const infoUrls = {
      model_name: generateUrl('InternetGatewayDevice.DeviceInfo.ModelName'),
      ssid: generateUrl('InternetGatewayDevice.LANDevice.1.WLANConfiguration.1.SSID'),
      connected: generateUrl('VirtualParameters.userconnected'),
      redaman: generateUrl('VirtualParameters.redaman'),
      temp: generateUrl('VirtualParameters.temp'),
      uptime: generateUrl('VirtualParameters.uptimeDevice'),
      username: generateUrl('VirtualParameters.pppUsername'),
      ipaddress: generateUrl('VirtualParameters.pppIP'),
      lokasi: generateUrl('_tags'),
      inform: `${config.genieacsBaseUrl}/devices?query=${encodeURIComponent(JSON.stringify({ _id: deviceId }))}&projection=_lastInform`
    };

    // Logging URL yang digunakan untuk masing-masing request
    Object.entries(infoUrls).forEach(([key, url]) => {
      console.log(`URL untuk ${key}: ${url}`);
    });

    const responses = await Promise.all(Object.values(infoUrls).map(url => axios.get(url).then(res => res.data)));

    const modelName = responses[0][0].InternetGatewayDevice.DeviceInfo.ModelName?._value || 'Model Name Not Found';
    const ssid = responses[1][0].InternetGatewayDevice.LANDevice['1'].WLANConfiguration['1'].SSID?._value || 'SSID Not Found';
    const connected = responses[2][0].VirtualParameters.userconnected?._value || 'User Connected Not Found';
    const redaman = responses[3][0].VirtualParameters.redaman?._value || 'Redaman Not Found';
    const temp = responses[4][0].VirtualParameters.temp?._value || 'Temperature Not Found';
    const uptime = responses[5][0].VirtualParameters.uptimeDevice?._value || 'Uptime Not Found';
    const username = responses[6][0].VirtualParameters.pppUsername?._value || 'Username Not Found';
    const ipaddress = responses[7][0].VirtualParameters.pppIP?._value || 'IP Address Not Found';
    const lokasi = responses[8][0]._tags || 'belum ditentukan';

    const informTimeStr = responses[9][0]._lastInform;
    const informTime = new Date(informTimeStr);
    const currentTime = new Date();

    const onlineThreshold = new Date(currentTime.getTime() - 5 * 60000); // 5 minutes
    const offlineThreshold = new Date(currentTime.getTime() - 24 * 60 * 60000); // 1 day

    let status;
    if (informTime > onlineThreshold) {
      status = "Online \u{1F7E2}";  // Green Circle (Unicode)
    } else if (informTime > offlineThreshold) {
      status = "Offline \u{1F534}"; // Red Circle (Unicode)
    } else {
      status = "Isolir \u{26AB}"; // Black Circle (Unicode)
    }

    return {
      modelName,
      ssid,
      connected,
      redaman,
      temp,
      uptime,
      username,
      ipaddress,
      lokasi,
      status
    };
  } catch (error) {
    console.error('Failed to fetch device data:', error);
    throw error;
  }
};


const rebootModem = async (identifier) => {
  try {
    if (!identifier) {
      return "Harap berikan device ID atau pppUsername. Penggunaan: /reboot <device_id atau pppUsername>";
    }

    // Coba untuk menemukan perangkat berdasarkan device_id
    let query = { _id: identifier };
    let queryUrl = `${config.genieacsBaseUrl}/devices?query=${encodeURIComponent(JSON.stringify(query))}`;
    console.log("Query URL untuk device_id:", queryUrl);

    let response = await axios.get(queryUrl);
    let devices = response.data;

    if (devices.length === 0) {
      // Jika tidak ditemukan, coba untuk menemukan berdasarkan pppUsername
      query = { "VirtualParameters.pppUsername": identifier };
      queryUrl = `${config.genieacsBaseUrl}/devices?query=${encodeURIComponent(JSON.stringify(query))}&projection=_id`;
      console.log("Query URL untuk pppUsername:", queryUrl);

      response = await axios.get(queryUrl);
      devices = response.data;
      if (devices.length === 0) {
        return `Tidak ada perangkat ditemukan untuk ${identifier}.`;
      }
    }

    const deviceId = devices[0]._id;
    // Encode deviceId if it contains '%'
    const encodedDeviceId = deviceId.includes('%') ? encodeURIComponent(deviceId) : deviceId;
    const url = `${config.genieacsBaseUrl}/devices/${encodedDeviceId}/tasks?connection_request`;
    const taskData = { name: 'reboot' };
    console.log("URL untuk reboot modem:", url);
    console.log("Data tugas yang dikirim:", taskData);

    response = await axios.post(url, taskData);

    if (response.status === 202) {
      return "Perintah reboot modem telah diterima dan sedang diproses.";
    } else if (response.status === 200) {
      return "Perintah reboot modem berhasil dikirim.";
    } else {
      return `Gagal mengirim perintah reboot modem. Kode status: ${response.status}`;
    }
  } catch (error) {
    console.error('Gagal mengirim perintah reboot modem:', error);
    return `Gagal mengirim perintah reboot modem: ${error.message}`;
  }
};



const refreshModem = async (identifier) => {
  try {
    // Coba untuk menemukan perangkat berdasarkan device_id
    let query = { _id: identifier };
    let queryUrl = `${config.genieacsBaseUrl}/devices?query=${encodeURIComponent(JSON.stringify(query))}`;
    console.log("Query URL untuk device_id:", queryUrl); // Log URL untuk debugging
    let response = await axios.get(queryUrl);
    let devices = response.data;

    if (devices.length === 0) {
      // Jika tidak ditemukan, coba untuk menemukan berdasarkan pppUsername
      query = { "VirtualParameters.pppUsername": identifier };
      queryUrl = `${config.genieacsBaseUrl}/devices?query=${encodeURIComponent(JSON.stringify(query))}&projection=_id`;
      console.log("Query URL untuk pppUsername:", queryUrl); // Log URL untuk debugging
      response = await axios.get(queryUrl);
      devices = response.data;

      if (devices.length === 0) {
        return `No device found for ${identifier}.`;
      }
    }

    const deviceId = devices[0]._id;
    // Encode deviceId if it contains '%'
    const encodedDeviceId = deviceId.includes('%') ? encodeURIComponent(deviceId) : deviceId;
    const url = `${config.genieacsBaseUrl}/devices/${encodedDeviceId}/tasks?timeout=3000&connection_request`;
    const taskData = {
      name: 'refreshObject',
      objectName: 'InternetGatewayDevice'
    };

    console.log("Query yang digunakan:", query);
    console.log("URL untuk refresh modem:", url);
    console.log("Data tugas yang dikirim:", taskData);

    const result = await axios.post(url, taskData);

    if (result.status === 202) {
      return "Modem refresh command has been accepted and is being processed.";
    } else if (result.status === 200) {
      return "Modem refresh command successfully sent.";
    } else {
      throw new Error(`Failed to send modem refresh command. Status code: ${result.status}`);
    }
  } catch (error) {
    console.error('Failed to send modem refresh command:', error);
    return `Failed to send modem refresh command: ${error.message}`;
  }
};


const getClientInfo = async (identifier) => {
  try {
    if (!identifier) {
      throw new Error("Please provide a device ID or pppUsername.");
    }

    // Mencoba menemukan perangkat berdasarkan ID perangkat
    let query = { _id: identifier };
    let queryUrl = `${config.genieacsBaseUrl}/devices?query=${encodeURIComponent(JSON.stringify(query))}`;
    console.log("Query yang digunakan:", query);
    let response = await axios.get(queryUrl);
    let devices = response.data;

    if (!devices.length) {
      // Jika tidak ditemukan, coba mencari berdasarkan pppUsername
      query = { "VirtualParameters.pppUsername": identifier };
      queryUrl = `${config.genieacsBaseUrl}/devices?query=${encodeURIComponent(JSON.stringify(query))}&projection=_id`;
      console.log("Query yang digunakan untuk pppUsername:", query);
      response = await axios.get(queryUrl);
      devices = response.data;
      if (!devices.length) {
        throw new Error(`No device found for ${identifier}.`);
      }
    }

    let deviceId = devices[0]._id;

    // Mengganti % dengan %25 pada deviceId
    deviceId = deviceId.replace(/%/g, '%25');

    // Mengirim permintaan refresh untuk memperbarui data perangkat
    const refreshUrl = `${config.genieacsBaseUrl}/devices/${deviceId}/tasks?connection_request`;
    await axios.post(refreshUrl, { name: "refreshObject", objectName: "" });
    console.log(`Mengirim permintaan refresh ke perangkat ${deviceId}`);

    // Menunggu beberapa saat agar perangkat memperbarui datanya
    await new Promise(resolve => setTimeout(resolve, 1000));

    // Mendapatkan informasi perangkat yang terhubung dengan projection yang sesuai
    const projection = [
      'InternetGatewayDevice.LANDevice.1.WLANConfiguration.1.AssociatedDevice',
      'InternetGatewayDevice.LANDevice.1.WLANConfiguration.2.AssociatedDevice',
      //'InternetGatewayDevice.LANDevice.1.WLANConfiguration.3.AssociatedDevice',
      //'InternetGatewayDevice.LANDevice.1.WLANConfiguration.4.AssociatedDevice',
      //'InternetGatewayDevice.LANDevice.1.WLANConfiguration.5.AssociatedDevice',
      //'InternetGatewayDevice.LANDevice.1.WLANConfiguration.6.AssociatedDevice',
      //'InternetGatewayDevice.LANDevice.1.WLANConfiguration.7.AssociatedDevice',
      //'InternetGatewayDevice.LANDevice.1.WLANConfiguration.8.AssociatedDevice'
    ].join(',');

    const clientInfoUrl = `${config.genieacsBaseUrl}/devices?query=${encodeURIComponent(JSON.stringify({ _id: deviceId }))}&projection=${encodeURIComponent(projection)}`;
    console.log("URL untuk mendapatkan informasi klien:", clientInfoUrl);
    const clientInfoResponse = await axios.get(clientInfoUrl);
    const clientInfoData = clientInfoResponse.data;

    // Mengumpulkan semua associated devices dari konfigurasi WLAN yang relevan
    let associatedDevices = [];
    for (const configKey in clientInfoData[0]?.InternetGatewayDevice?.LANDevice?.['1']?.WLANConfiguration) {
      const wlanConfig = clientInfoData[0].InternetGatewayDevice.LANDevice['1'].WLANConfiguration[configKey];
      if (wlanConfig?.AssociatedDevice) {
        for (const deviceKey in wlanConfig.AssociatedDevice) {
          associatedDevices.push(wlanConfig.AssociatedDevice[deviceKey]);
        }
      }
    }

    // Tentukan parameter berdasarkan vendor jika tersedia
    const getDescription = (device) => device.X_HW_AssociatedDevicedescriptions?._value ||
      device.X_ZTE_COM_AssociatedDeviceName?._value ||
      device.X_ZTE_COM_DeviceName?._value ||
      'Deskripsi Tidak Tersedia';
    const getMac = (device) => device.AssociatedDeviceMACAddress?._value || 'MAC Tidak Tersedia';
    const getIp = (device) => device.AssociatedDeviceIPAddress?._value || 'IP Tidak Tersedia';
    const getSignalQuality = (device) => device.X_HW_SingalQuality?._value || 'Kualitas Sinyal Tidak Tersedia';



    // Mengambil setiap perangkat yang terhubung dari objek AssociatedDevice
    let deviceIndex = 1;
    const connectedClients = associatedDevices
      .map((device) => {
        const ip = getIp(device);
        const mac = getMac(device);
        const signalQuality = getSignalQuality(device);
        const description = getDescription(device);

        // Hanya menampilkan data yang tersedia
        if (description === 'Deskripsi Tidak Tersedia' &&
          mac === 'MAC Tidak Tersedia' &&
          ip === 'IP Tidak Tersedia' &&
          signalQuality === 'Kualitas Sinyal Tidak Tersedia') {
          return null; // Tidak termasuk perangkat dengan semua data tidak tersedia
        }

        let details = `No: ${deviceIndex++}\n`;
        if (description) details += `Nama Perangkat: ${description}\n`;
        if (mac) details += `MAC: ${mac}\n`;
        if (ip) details += `IP: ${ip}\n`;
        if (signalQuality) details += `Signal: ${signalQuality}\n`;

        return details;
      })
      .filter(detail => detail) // Filter out null values
      .join('\n\n'); // Pisahkan setiap detail perangkat dengan dua baris baru

    const validDeviceCount = deviceIndex - 1; // Jumlah perangkat yang valid

    return `Perangkat Terhubung: ${validDeviceCount}\n\n${connectedClients || 'Tidak ada perangkat yang terhubung'}`;
  } catch (error) {
    console.error('Gagal mendapatkan informasi klien:', error);
    throw error;
  }
};


module.exports = {
  setPassword,
  setSSIDName,
  getDeviceInfo,
  getDeviceByIdentifier,
  getSSIDInfo,
  rebootModem,
  refreshModem,
  getClientInfo,
  ubahSSID,
  ubahPass
};
